<?php
require_once(realpath(dirname(__FILE__) . "/../tools/rest.php"));

class Payment extends REST{

    private $mysqli = NULL;
    private $db = NULL;
    private $product_order_detail   = NULL;
    private $config 			    = NULL;
    private $payment_conf           = NULL;
    private $currency               = NULL;
    private $PAYPAL_URL_SANDBOX     = "https://api-m.sandbox.paypal.com";
    private $PAYPAL_URL_LIVE        = "https://api-m.paypal.com";

    private $MIDTRANS_URL_SANDBOX   = "https://app.sandbox.midtrans.com";
    private $MIDTRANS_URL_LIVE      = "https://app.midtrans.com";

    public function __construct($db) {
        parent::__construct();
        $this->db = $db;
        $this->mysqli = $db->mysqli;
        $this->product_order_detail = new ProductOrderDetail($this->db);
        $this->config = new Config($this->db);

        $this->payment_conf = $this->config->findByGroupPlain('PAYMENT');
        $this->currency = $this->config->findByCodePlain('GENERAL')['currency'];
    }

    public function showPage(){
        if($this->get_request_method() != "GET") $this->response('',406);
        if(!isset($this->_request['id']) && !isset($this->_request['code']))$this->responseInvalidParam();
        $product_order_id = isset($this->_request['id']) ? (int)$this->_request['id'] : 0;
        $product_order_code = isset($this->_request['code']) ? $this->_request['code'] : '';
        if($product_order_id > 0){
            $product_order = $this->db->get_one("SELECT * FROM product_order po WHERE po.id=$product_order_id");
        } else {
            $product_order = $this->db->get_one("SELECT * FROM product_order po WHERE po.code='$product_order_code'");
            if(sizeof($product_order) != 0) $product_order_id = $product_order['id'];
        }

        $page_type = 'READY';
        $msg = '';

        if(sizeof($product_order) == 0){
            $page_type = 'NOT_FOUND';
            $msg = 'Order not found';
        } else if($product_order['payment_status'] == 'PAID'){
            $page_type = 'PAID';
            $msg = 'Order already paid';
        } else if($product_order['payment_status'] == 'PENDING'){
            $page_type = 'PENDING';
            $msg = 'Waiting for payment to be completed';
        } else if($product_order['payment_status'] == 'EXPIRED'){
            $page_type = 'EXPIRED';
            $msg = 'Payment has expired';
        } else if($product_order['payment_status'] == 'DENIED'){
            $page_type = 'DENIED';
            $msg = 'Payment canceled or denied';
        } else if($product_order['payment_status'] == 'REFUND'){
            $page_type = 'REFUND';
            $msg = 'Order has been refunded';
        } else {
            $order_detail = $this->product_order_detail->findAllByOrderIdPlain($product_order_id);
            $resp_stock = $this->product_order_detail->checkAvailableProductOrderDetail($order_detail);
            $product_order['amount'] = $product_order['total_fees'] . ' ' . $this->currency;
            if($resp_stock['status'] == 'failed'){
                $page_type = 'OUT_OF_STOCK';
                $msg = str_replace("\\n", '<br>', $resp_stock['msg']);
            }
            $product_order['amount'] = $product_order['total_fees'] . ' ' . $this->currency;
        }

        $payment_midtrans   = json_decode($this->payment_conf['PAYMENT_MIDTRANS'], true);
        $payment_paypal     = json_decode($this->payment_conf['PAYMENT_PAYPAL'], true);
        $payment_razorpay   = json_decode($this->payment_conf['PAYMENT_RAZORPAY'], true);
        $payment_bank       = json_decode($this->payment_conf['PAYMENT_BANK'], true);

        if($page_type ==  'READY'){
            $template = file_get_contents(realpath(dirname(__FILE__) . "/template.html"));
        } else {
            $template = file_get_contents(realpath(dirname(__FILE__) . "/status.html"));
        }

        $product_order['currency']          = $this->currency;

        $product_order['midtrans_active']     = $payment_midtrans['active'];
        $product_order['midtrans_client_key'] = $payment_midtrans['client_key'];
        $product_order['midtrans_js_url']   = $this->MIDTRANS_URL_SANDBOX;
        if($payment_midtrans['mode']        == 'LIVE') $product_order['midtrans_js_url'] = $this->MIDTRANS_URL_LIVE;
        $product_order['midtrans_js_url']   = $product_order['midtrans_js_url'] . '/snap/snap.js';

        $product_order['paypal_active']     = $payment_paypal['active'];
        $product_order['paypal_client_id']  = $payment_paypal['client_id'];
        $product_order['razorpay_active']   = $payment_razorpay['active'];
        $product_order['razorpay_key_id']   = $payment_razorpay['key_id'];
        $product_order['bank_active']       = $payment_bank['active'];
        $product_order['bank_instruction']  = addslashes($payment_bank['instruction']);

        $product_order['page_type']         = $page_type;
        $product_order['msg']               = $msg;
        $product_order = array_reverse($product_order);
        foreach ($product_order as $key => $value) {
            $tagToReplace = "[@$key]";
            $template = str_replace($tagToReplace, $value, $template);
        }
        echo $template;
        exit;
    }

    public function validateCompletePayment($order){
        $payment = $order['payment'];
        if($payment == 'MIDTRANS'){
            return $this->validateMidtrans($order);
        } else if($payment == 'PAYPAL'){
            return $this->validatePaypal($order);
        } else if($payment == 'RAZORPAY'){
            return $this->validateRazorPay($order);
        } else {
            return null;
        }
    }

    public function validateMidtrans($order){
        $midtrans_conf = json_decode($this->payment_conf['PAYMENT_MIDTRANS'], true);

        $url = $this->MIDTRANS_URL_SANDBOX;
        if($midtrans_conf['mode'] ==  'LIVE') $url = $this->MIDTRANS_URL_LIVE;
        $url = $url . "/v2/" . $order['code'] . "/status";
        $url = str_replace('app.', 'api.', $url);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');

        $headers = array();
        $headers[] = 'Authorization: Basic '.base64_encode($midtrans_conf['server_key']);
        $headers[] = 'Accept: application/json';
        $headers[] = 'Content-Type: application/json';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        curl_close($ch);
        $resp = json_decode($result, true);
        if(!array_key_exists('transaction_status', $resp)) return null;
        return $resp;
    }

    public function validatePaypal($order){
        $paypal_conf = json_decode($this->payment_conf['PAYMENT_PAYPAL'], true);
        $url = $this->PAYPAL_URL_SANDBOX;
        if($paypal_conf['mode'] ==  'LIVE') {
            $url = $this->PAYPAL_URL_LIVE;
        }
        $token = $this->getPaypalToken();
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url.'/v2/checkout/orders/'.$order['payment_data']);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');


        $headers = array();
        $headers[] = 'Authorization: Bearer '.$token;
        $headers[] = 'Cache-Control: no-cache';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        curl_close($ch);
        $resp = json_decode($result, true);
        if(!array_key_exists('status', $resp) || !array_key_exists('purchase_units', $resp)) return null;
        return $resp;
    }

    public function validateRazorPay($order){
        $razorpay_conf = json_decode($this->payment_conf['PAYMENT_RAZORPAY'], true);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.razorpay.com/v1/orders/'.$order['payment_data']);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_USERPWD, $razorpay_conf['key_id'] . ':' . $razorpay_conf['key_secret']);

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        curl_close($ch);
        $resp = json_decode($result, true);
        if(!array_key_exists('receipt', $resp) || !array_key_exists('status', $resp)) return null;
        return $resp;
    }

    public function getPaypalToken(){
        $paypal_conf = json_decode($this->payment_conf['PAYMENT_PAYPAL'], true);
        $url = $this->PAYPAL_URL_SANDBOX;
        if($paypal_conf['mode'] ==  'LIVE') $url = $this->PAYPAL_URL_LIVE;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url.'/v1/oauth2/token');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, "grant_type=client_credentials");
        curl_setopt($ch, CURLOPT_USERPWD, $paypal_conf['client_id'] . ':' . $paypal_conf['secret']);

        $headers = array();
        $headers[] = 'Accept: application/json';
        $headers[] = 'Accept-Language: en_US';
        $headers[] = 'Content-Type: application/x-www-form-urlencoded';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        curl_close($ch);
        $resp = json_decode($result, true);
        if(is_array($resp) && !array_key_exists("access_token",$resp)){
            return null;
        }
        return $resp['access_token'];
    }
    
    public function generateMidtransOrderId($order){
        $midtrans_conf = json_decode($this->payment_conf['PAYMENT_MIDTRANS'], true);
        $url = $this->MIDTRANS_URL_SANDBOX;
        if($midtrans_conf['mode'] ==  'LIVE') $url = $this->MIDTRANS_URL_LIVE;

        $data = array (
            'transaction_details' => array (
                'order_id' => $order['code'],
                'gross_amount' => (int) $order['total_fees'],
            ),
            'credit_card' => array (
                'secure' => true,
            ),
            'expiry' => array (
                "unit" => "hour",
                "duration" => 24
            )
        );

        $notification_url = $this->getAPIUrl() . 'webhookMidtrans';

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url . '/snap/v1/transactions');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $this->json($data));

        $headers = array();
        $headers[] = 'Authorization: Basic '.base64_encode($midtrans_conf['server_key']);
        $headers[] = 'Accept: application/json';
        $headers[] = 'Content-Type: application/json';
        $headers[] = 'X-Append-Notification: ' . $notification_url;
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        $resp = json_decode($result, true);
        if (curl_errno($ch)) {
            return array('status' => 'failed', 'msg' => 'UNKNOWN ERROR');
        }
        curl_close($ch);
        if(array_key_exists("token", $resp)){
            return array('status' => 'success', 'msg' => '', 'data' => $resp['token']);
        } else if(array_key_exists("error_messages", $resp) && sizeof($resp['error_messages']) > 0){
            return array('status' => 'failed', 'msg' => $resp['error_messages'][0]);
        } else {
            return array('status' => 'failed', 'msg' => 'UNKNOWN ERROR');
        }
    }

    public function generatePaypalOrderId($order){
        $paypal_conf = json_decode($this->payment_conf['PAYMENT_PAYPAL'], true);
        $url = $this->PAYPAL_URL_SANDBOX;
        if($paypal_conf['mode'] ==  'LIVE') $url = $this->PAYPAL_URL_LIVE;

        $token = $this->getPaypalToken();
        $data = array(
            'intent' => 'CAPTURE', 'purchase_units' => array(
                array(
                    'description' => $order['code'], 'custom_id' => $order['code'],
                    'amount' => array('currency_code' => $this->currency, 'value' => $order['total_fees'])
                )
            ),
            'application_context' => array(
                'shipping_preference' => 'NO_SHIPPING', 'brand_name' => 'Markeet App'
            )
        );

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $url.'/v2/checkout/orders');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $this->json($data));

        $headers = array();
        $headers[] = 'Authorization: Bearer '.$token;
        $headers[] = 'Content-Type: application/json';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        $resp = json_decode($result, true);
        if (curl_errno($ch)) {
            return array('status' => 'failed', 'msg' => 'UNKNOWN ERROR');
        }
        curl_close($ch);
        if(array_key_exists("id", $resp)){
            return array('status' => 'success', 'msg' => '', 'data' => $resp['id']);
        } else if(array_key_exists("details", $resp) && sizeof($resp['details']) > 0){
            return array('status' => 'failed', 'msg' => $resp['details'][0]['description']);
        } else {
            return array('status' => 'failed', 'msg' => 'UNKNOWN ERROR');
        }
    }

    public function generateRazorPayOrderId($order){
        $razorpay_conf = json_decode($this->payment_conf['PAYMENT_RAZORPAY'], true);
        $amount = $order['total_fees'] * 100;
        $data = array('amount' => $amount, 'currency' => $this->currency, 'receipt' => $order['code']);

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, 'https://api.razorpay.com/v1/orders');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $this->json($data));
        curl_setopt($ch, CURLOPT_USERPWD, $razorpay_conf['key_id'] . ':' . $razorpay_conf['key_secret']);

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        $resp = json_decode($result, true);
        if (curl_errno($ch)) {
            return array('status' => 'failed', 'msg' => 'UNKNOWN ERROR');
        }
        curl_close($ch);
        if(array_key_exists("id", $resp)){
            return array('status' => 'success', 'msg' => '', 'data' => $resp['id']);
        } else {
            return array('status' => 'failed', 'msg' => $resp['error']['description']);
        }
    }

    public function getAPIUrl(){
        $link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http");
        $link = $link .  "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
        $arr_link = explode("/services/", $link);
        return $arr_link[0] . '/services/';
    }

}
?>