package com.app.markeet.activity;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.text.Html;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.text.HtmlCompat;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.app.markeet.AppConfig;
import com.app.markeet.R;
import com.app.markeet.adapter.AdapterListener;
import com.app.markeet.adapter.AdapterProduct;
import com.app.markeet.connection.API;
import com.app.markeet.connection.RestAdapter;
import com.app.markeet.connection.callbacks.RespProduct;
import com.app.markeet.data.Constant;
import com.app.markeet.data.DatabaseHandler;
import com.app.markeet.data.ThisApp;
import com.app.markeet.model.Category;
import com.app.markeet.model.Product;
import com.app.markeet.model.SortBy;
import com.app.markeet.utils.Tools;

import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class ActivityCategoryDetails extends AppCompatActivity {
    private static final String EXTRA_OBJECT = "key.EXTRA_OBJECT";

    // activity transition
    public static void navigate(Activity activity, Category obj) {
        Intent i = new Intent(activity, ActivityCategoryDetails.class);
        i.putExtra(EXTRA_OBJECT, obj);
        activity.startActivity(i);
    }

    // extra obj
    private Category category;

    private View cart_badge, lytColor;
    private SwipeRefreshLayout swipe_refresh;
    private Call<RespProduct> callbackCall = null;

    private RecyclerView recyclerView;
    private AdapterProduct mAdapter;
    private DatabaseHandler db;

    private int post_total = 0;
    private SortBy sort;
    private int sort_selected = 0;

    private int cart_count = -1;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_category_details);
        category = (Category) getIntent().getSerializableExtra(EXTRA_OBJECT);
        sort = ThisApp.get().getSorts().get(sort_selected);
        db = new DatabaseHandler(this);

        initComponent();

        displayCategoryData(category);

        requestAction(1);

        Tools.RTLMode(getWindow());
    }

    private void initComponent() {
        swipe_refresh = (SwipeRefreshLayout) findViewById(R.id.swipe_refresh_layout);
        recyclerView = (RecyclerView) findViewById(R.id.recycler_view);
        recyclerView.setLayoutManager(new GridLayoutManager(this, Tools.getGridSpanCount(this)));
        cart_badge = findViewById(R.id.cart_badge);
        lytColor = findViewById(R.id.lyt_color);

        //set data and list adapter
        mAdapter = new AdapterProduct(this, recyclerView, AppConfig.general.product_per_request);
        recyclerView.setAdapter(mAdapter);

        // detect when scroll reach bottom
        mAdapter.setListener(new AdapterListener<Product>() {
            @Override
            public void onClick(View view, String type, Product obj, int position) {
                super.onClick(view, type, obj, position);
                ActivityProductDetails.navigate(ActivityCategoryDetails.this, obj.id, false);
            }

            @Override
            public void onLoadMore(int current_page) {
                super.onLoadMore(current_page);
                if (post_total > mAdapter.getItemCount() && current_page != 0) {
                    int next_page = current_page + 1;
                    requestAction(next_page);
                } else {
                    mAdapter.setLoaded();
                }
            }
        });

        // on swipe list
        swipe_refresh.setOnRefreshListener(() -> {
            if (callbackCall != null && callbackCall.isExecuted()) callbackCall.cancel();
            mAdapter.resetListData();
            requestAction(1);
        });

        (findViewById(R.id.toolbar_menu_back)).setOnClickListener(v -> onBackPressed());

        (findViewById(R.id.toolbar_menu_sort)).setOnClickListener(v -> showDialogProductSort());

        (findViewById(R.id.toolbar_menu_cart)).setOnClickListener(v -> ActivityShoppingCart.navigate(this));

        (findViewById(R.id.toolbar_menu_search)).setOnClickListener(v -> ActivitySearch.navigate(ActivityCategoryDetails.this, category));
    }

    private void displayCategoryData(Category c) {
        TextView tvName = ((TextView) findViewById(R.id.name));
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            tvName.setText(Html.fromHtml(c.name, Html.FROM_HTML_MODE_COMPACT));
        } else {
            tvName.setText(HtmlCompat.fromHtml(c.name, HtmlCompat.FROM_HTML_MODE_LEGACY));
        }

        ((TextView) findViewById(R.id.brief)).setText(c.brief);
        ImageView icon = (ImageView) findViewById(R.id.icon);

        Tools.displayImageOriginal(this, icon, Constant.getURLimgCategory(c.icon));
        if (AppConfig.general.tint_category_icon) {
            icon.setColorFilter(getResources().getColor(R.color.textIconPrimary));
        }

        lytColor.setBackgroundColor(Color.parseColor(c.color));

    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
    }

    @Override
    protected void onResume() {
        super.onResume();
        int new_cart_count = db.getActiveCartSize();
        if (new_cart_count != cart_count) {
            cart_count = new_cart_count;
        }
        setupBadge();
    }


    private void displayApiResult(final List<Product> items) {
        mAdapter.insertData(items);
        swipeProgress(false);
        if (items.size() == 0) showNoItemView(true);
    }

    private void requestListProduct(final int page_no) {
        API api = RestAdapter.createAPI();
        callbackCall = api.getListProduct(page_no, AppConfig.general.product_per_request, null, category.id, sort.column, sort.order);
        callbackCall.enqueue(new Callback<RespProduct>() {
            @Override
            public void onResponse(Call<RespProduct> call, Response<RespProduct> response) {
                RespProduct resp = response.body();
                if (resp != null && resp.status.equals("success")) {
                    post_total = resp.count_total;
                    displayApiResult(resp.products);
                } else {
                    onFailRequest();
                }
            }

            @Override
            public void onFailure(Call<RespProduct> call, Throwable t) {
                if (!call.isCanceled()) onFailRequest();
            }

        });
    }

    private void onFailRequest() {
        swipeProgress(false);
        if (Tools.isConnect(this)) {
            mAdapter.setLoadingOrFailed(getString(R.string.failed_text));
        } else {
            mAdapter.setLoadingOrFailed(getString(R.string.no_internet_text));
        }
    }

    private void requestAction(final int page_no) {
        showNoItemView(false);
        if (page_no == 1) {
            mAdapter.resetListData();
            swipeProgress(true);
        } else {
            mAdapter.setLoading();
        }
        new Handler(getMainLooper()).postDelayed(() -> requestListProduct(page_no), 1000);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        swipeProgress(false);
        if (callbackCall != null && callbackCall.isExecuted()) {
            callbackCall.cancel();
        }
    }

    private void showNoItemView(boolean show) {
        View lyt_no_item = (View) findViewById(R.id.lyt_no_item);
        if (show) {
            recyclerView.setVisibility(View.GONE);
            lyt_no_item.setVisibility(View.VISIBLE);
        } else {
            recyclerView.setVisibility(View.VISIBLE);
            lyt_no_item.setVisibility(View.GONE);
        }
    }

    private void swipeProgress(final boolean show) {
        if (!show) {
            swipe_refresh.setRefreshing(show);
            return;
        }
        swipe_refresh.post(() -> swipe_refresh.setRefreshing(show));
    }

    private void showDialogProductSort() {
        final List<SortBy> sorts = ThisApp.get().getSorts();
        String[] sort_choice = new String[sorts.size()];
        for (int i = 0; i < sorts.size(); i++) sort_choice[i] = sorts.get(i).label;
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.title_dialog_sort);
        builder.setSingleChoiceItems(sort_choice, sort_selected, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                sort_selected = i;
                sort = sorts.get(sort_selected);
                dialogInterface.dismiss();
                initComponent();
                requestAction(1);
            }
        });
        builder.show();
    }

    private void setupBadge() {
        if (cart_badge == null) return;
        if (cart_count == 0) {
            cart_badge.setVisibility(View.GONE);
        } else {
            cart_badge.setVisibility(View.VISIBLE);
            String count_txt = cart_count + "";
            if (cart_count > 9) count_txt = "9+";
            ((TextView) cart_badge.findViewById(R.id.counter)).setText(count_txt);
        }
    }

}
