package com.app.markeet.activity;

import android.app.Dialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;

import androidx.annotation.Nullable;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.view.GravityCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;

import com.app.markeet.AppConfig;
import com.app.markeet.R;
import com.app.markeet.advertise.AdNetworkHelper;
import com.app.markeet.data.DatabaseHandler;
import com.app.markeet.data.SharedPref;
import com.app.markeet.data.ThisApp;
import com.app.markeet.databinding.ActivityMainBinding;
import com.app.markeet.fragment.FragmentCategory;
import com.app.markeet.fragment.FragmentHome;
import com.app.markeet.fragment.FragmentNewsInfo;
import com.app.markeet.fragment.FragmentProfile;
import com.app.markeet.utils.PermissionUtil;
import com.app.markeet.utils.Tools;

public class ActivityMain extends AppCompatActivity {

    private ActivityMainBinding binding;

    private FragmentHome fragmentHome;
    private FragmentCategory fragmentCategory;
    private FragmentNewsInfo fragmentNewsInfo;
    private FragmentProfile fragmentProfile;

    private Fragment selectedFragment;
    private FragmentManager fm;

    private DatabaseHandler db;
    private SharedPref sharedPref;
    private Dialog dialog_failed = null;
    public boolean category_load = false, news_load = false;

    static ActivityMain activityMain;

    public static ActivityMain getInstance() {
        return activityMain;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        activityMain = this;
        db = new DatabaseHandler(this);
        sharedPref = new SharedPref(this);

        initDrawerMenu();
        initComponent();
        prepareAds();


        // set default fragment
        selectedFragment = fragmentHome;
        displayFragment(R.id.nav_home, getString(R.string.menu_home));

        // launch instruction when first launch
        if (sharedPref.isFirstLaunch()) {
            startActivity(new Intent(this, ActivityInstruction.class));
            sharedPref.setFirstLaunch(false);
        }
    }


    private void initComponent() {
        setSupportActionBar(binding.toolbar);
        ActionBar actionBar = getSupportActionBar();
        actionBar.setTitle(null);
        fm = getSupportFragmentManager();

        fragmentHome = FragmentHome.instance();
        fragmentCategory = FragmentCategory.instance();
        fragmentNewsInfo = FragmentNewsInfo.instance();
        fragmentProfile = FragmentProfile.instance();

        fm.beginTransaction().add(R.id.frame_layout, fragmentHome, getString(R.string.menu_home)).commit();
        fm.beginTransaction().add(R.id.frame_layout, fragmentCategory, getString(R.string.menu_category)).hide(fragmentCategory).commit();
        fm.beginTransaction().add(R.id.frame_layout, fragmentNewsInfo, getString(R.string.menu_news_info)).hide(fragmentNewsInfo).commit();
        fm.beginTransaction().add(R.id.frame_layout, fragmentProfile, getString(R.string.menu_profile)).hide(fragmentProfile).commit();

        binding.navigation.setOnItemSelectedListener(item -> {
            displayFragment(item.getItemId(), item.getTitle().toString());
            showInterstitialAd();
            return true;
        });

        // listener for hide toolbar and bottom nav
        binding.appbarLayout.addOnOffsetChangedListener((appBarLayout, verticalOffset) -> {
            int offset = Math.abs(verticalOffset), range = appBarLayout.getTotalScrollRange();
            int heightMax = binding.lytBarContent.getHeight();
            if (range <= 0) return;
            int translationY = (offset * heightMax) / range;
            binding.lytBar.setTranslationY(translationY);
        });

        binding.toolbarMenuMore.setOnClickListener(v -> {
            binding.drawerLayout.openDrawer(GravityCompat.START);
        });


        binding.toolbarMenuCart.setOnClickListener(v -> {
            ActivityShoppingCart.navigate(this);
        });

        binding.toolbarMenuSearch.setOnClickListener(v -> {
            ActivitySearch.navigate(this);
        });

    }

    public void displayFragment(int id, String title) {
        if (id == R.id.nav_home) {
            fm.beginTransaction().hide(selectedFragment).show(fragmentHome).commit();
            selectedFragment = fragmentHome;
        } else if (id == R.id.nav_categories) {
            fm.beginTransaction().hide(selectedFragment).show(fragmentCategory).commit();
            selectedFragment = fragmentCategory;
        } else if (id == R.id.nav_news_info) {
            fm.beginTransaction().hide(selectedFragment).show(fragmentNewsInfo).commit();
            selectedFragment = fragmentNewsInfo;
        } else if (id == R.id.nav_profile) {
            fm.beginTransaction().hide(selectedFragment).show(fragmentProfile).commit();
            selectedFragment = fragmentProfile;
        }
        binding.title.setText(title);
        binding.navigation.getMenu().findItem(id).setChecked(true);
    }

    private void initDrawerMenu() {
        binding.lytDrawerMenu.actionShoppingCart.setOnClickListener(view -> {
            ActivityShoppingCart.navigate(this);
        });

        binding.lytDrawerMenu.actionHistory.setOnClickListener(view -> {
            ActivityOrderHistory.navigate(this);
        });

        binding.lytDrawerMenu.actionWhishlist.setOnClickListener(view -> {
            ActivityWishlist.navigate(this);
        });

        binding.lytDrawerMenu.actionNotification.setOnClickListener(view -> {
            ActivityNotification.navigate(this);
        });

        binding.lytDrawerMenu.actionInstruction.setOnClickListener(view -> {
            ActivityInstruction.navigate(this);
        });

        binding.lytDrawerMenu.actionContactUs.setOnClickListener(view -> {
            Tools.directLinkToCustomTabs(this, AppConfig.general.contact_us_url);
        });

        binding.lytDrawerMenu.actionRate.setOnClickListener(view -> {
            Tools.rateAction(this);
        });

        binding.lytDrawerMenu.actionAboutApp.setOnClickListener(view -> {
            Tools.showDialogAbout(this);
        });

        binding.lytDrawerMenu.version.setText(Tools.getVersionName(this));
    }

    private void updateNavCounter() {
        int cart_count = db.getActiveCartSize();
        binding.lytDrawerMenu.cartCount.setText(String.valueOf(cart_count));

        // update wishlist counter
        int wishlist_count = db.getWishlistSize();
        binding.lytDrawerMenu.whishlistCount.setText(String.valueOf(wishlist_count));

        // update notification counter
        int notif_count = db.getUnreadNotificationSize();
        if (notif_count > 0) {
            binding.lytDrawerMenu.read.setVisibility(View.VISIBLE);
        } else {
            binding.lytDrawerMenu.read.setVisibility(View.GONE);
        }
    }

    private void setupCartBadge() {
        int cart_count = db.getActiveCartSize();
        if (cart_count == 0) {
            binding.cartBadge.setVisibility(View.GONE);
        } else {
            binding.cartBadge.setVisibility(View.VISIBLE);
            String count_txt = cart_count + "";
            if (cart_count > 9) count_txt = "9+";
            binding.counter.setText(count_txt);
        }
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        PermissionUtil.checkAndRequestNotification(this);
        ThisApp.get().obtainAndSubmitToServer();
    }

    @Override
    protected void onResume() {
        super.onResume();
        updateNavCounter();
        setupCartBadge();
    }

    static boolean active = false;

    @Override
    public void onStart() {
        super.onStart();
        active = true;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        active = false;
    }

    private AdNetworkHelper adNetworkHelper;

    private void prepareAds() {
        adNetworkHelper = new AdNetworkHelper(this);
        adNetworkHelper.init();
        adNetworkHelper.updateConsentStatus();
        adNetworkHelper.loadBannerAd(AppConfig.ads.ad_main_banner);
        adNetworkHelper.loadInterstitialAd(AppConfig.ads.ad_main_interstitial);
    }

    public void showInterstitialAd() {
        adNetworkHelper.showInterstitialAd(AppConfig.ads.ad_main_interstitial);
    }

}
