package com.app.markeet.activity;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.app.markeet.AppConfig;
import com.app.markeet.R;
import com.app.markeet.adapter.AdapterNotification;
import com.app.markeet.data.DatabaseHandler;
import com.app.markeet.model.Notification;
import com.google.android.material.snackbar.Snackbar;

import java.util.ArrayList;
import java.util.List;

public class ActivityNotification extends AppCompatActivity {

    public static void navigate(Activity activity) {
        Intent i = new Intent(activity, ActivityNotification.class);
        activity.startActivity(i);
    }

    public View parent_view;
    private RecyclerView recyclerView;
    private DatabaseHandler db;
    public AdapterNotification adapter;
    static ActivityNotification activityNotification;

    public static ActivityNotification getInstance() {
        return activityNotification;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_notifications);
        activityNotification = this;

        db = new DatabaseHandler(this);

        iniComponent();
    }


    private void iniComponent() {
        parent_view = findViewById(android.R.id.content);

        (findViewById(R.id.toolbar_menu_back)).setOnClickListener(v -> onBackPressed());
        (findViewById(R.id.toolbar_menu_delete)).setOnClickListener(v -> {
            if (adapter.getItemCount() == 0) {
                Snackbar.make(parent_view, R.string.msg_notif_empty, Snackbar.LENGTH_SHORT).show();
                return;
            }
            dialogDeleteConfirmation();
            dialogDeleteConfirmation();
        });

        recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        //set data and list adapter
        adapter = new AdapterNotification(this, recyclerView, new ArrayList<Notification>());
        recyclerView.setAdapter(adapter);
        recyclerView.setNestedScrollingEnabled(false);

        adapter.setOnItemClickListener((view, obj, pos) -> {
            obj.read = true;
            ActivityDialogNotification.navigate(ActivityNotification.this, obj, false, pos);
        });

        startLoadMoreAdapter();
    }

    @Override
    protected void onResume() {
        adapter.notifyDataSetChanged();
        super.onResume();
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
    }

    public void dialogDeleteConfirmation() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.title_delete_confirm);
        builder.setMessage(getString(R.string.content_delete_confirm) + getString(R.string.title_activity_notifications));
        builder.setPositiveButton(R.string.YES, (di, i) -> {
            di.dismiss();
            db.deleteNotification();
            startLoadMoreAdapter();
            Snackbar.make(parent_view, R.string.delete_success, Snackbar.LENGTH_SHORT).show();
        });
        builder.setNegativeButton(R.string.CANCEL, null);
        builder.show();
    }

    private void startLoadMoreAdapter() {
        adapter.resetListData();
        List<Notification> items = db.getNotificationsByPage(AppConfig.general.notification_page, 0);
        adapter.insertData(items);
        showNoItemView();
        final int item_count = (int) db.getNotificationSize();
        // detect when scroll reach bottom
        adapter.setOnLoadMoreListener(current_page -> {
            if (item_count > adapter.getItemCount() && current_page != 0) {
                displayDataByPage(current_page);
            } else {
                adapter.setLoaded();
            }
        });
    }

    private void displayDataByPage(final int next_page) {
        adapter.setLoading();
        new Handler().postDelayed(() -> {
            List<Notification> items = db.getNotificationsByPage(AppConfig.general.notification_page, (next_page * AppConfig.general.notification_page));
            adapter.insertData(items);
            showNoItemView();
        }, 500);
    }

    private void showNoItemView() {
        View lyt_no_item = (View) findViewById(R.id.lyt_no_item);
        if (adapter.getItemCount() == 0) {
            lyt_no_item.setVisibility(View.VISIBLE);
        } else {
            lyt_no_item.setVisibility(View.GONE);
        }
    }
}
