package com.app.markeet.activity;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Paint;
import android.os.Bundle;
import android.os.Handler;
import android.text.Html;
import android.util.Log;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebView;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;
import androidx.viewpager.widget.ViewPager;

import com.app.markeet.AppConfig;
import com.app.markeet.R;
import com.app.markeet.adapter.AdapterProductImage;
import com.app.markeet.advertise.AdNetworkHelper;
import com.app.markeet.connection.API;
import com.app.markeet.connection.RestAdapter;
import com.app.markeet.connection.callbacks.RespProductDetails;
import com.app.markeet.data.Constant;
import com.app.markeet.data.DatabaseHandler;
import com.app.markeet.data.SharedPref;
import com.app.markeet.model.Cart;
import com.app.markeet.model.Category;
import com.app.markeet.model.Product;
import com.app.markeet.model.ProductImage;
import com.app.markeet.model.Wishlist;
import com.app.markeet.utils.NetworkCheck;
import com.app.markeet.utils.Tools;
import com.balysv.materialripple.MaterialRippleLayout;
import com.google.android.flexbox.FlexboxLayout;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class ActivityProductDetails extends AppCompatActivity {

    private static final String EXTRA_OBJECT_ID = "key.EXTRA_OBJECT_ID";
    private static final String EXTRA_FROM_NOTIF = "key.EXTRA_FROM_NOTIF";

    // activity transition
    public static void navigate(Activity activity, Long id, Boolean from_notif) {
        Intent i = navigateBase(activity, id, from_notif);
        activity.startActivity(i);
    }

    public static Intent navigateBase(Context context, Long id, Boolean from_notif) {
        Intent i = new Intent(context, ActivityProductDetails.class);
        i.putExtra(EXTRA_OBJECT_ID, id);
        i.putExtra(EXTRA_FROM_NOTIF, from_notif);
        return i;
    }

    private Long product_id;
    private Boolean from_notif;

    // extra obj
    private Product product;

    private ImageView wishlist_menu;
    private boolean flag_wishlist = false;
    private boolean flag_cart = false;
    private DatabaseHandler db;

    private Call<RespProductDetails> callbackCall = null;
    private View parent_view;
    private SwipeRefreshLayout swipe_refresh;
    private MaterialRippleLayout lyt_add_cart;
    private TextView tv_add_cart;
    private WebView webview = null;
    private SharedPref sharedPref;

    private View cart_badge;
    private int cart_count = -1;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_product_details);

        product_id = getIntent().getLongExtra(EXTRA_OBJECT_ID, -1L);
        from_notif = getIntent().getBooleanExtra(EXTRA_FROM_NOTIF, false);

        db = new DatabaseHandler(this);
        sharedPref = new SharedPref(this);

        initComponent();
        requestAction();
        prepareAds();
    }

    private void initComponent() {
        parent_view = findViewById(android.R.id.content);
        swipe_refresh = findViewById(R.id.swipe_refresh_layout);
        lyt_add_cart = findViewById(R.id.lyt_add_cart);
        tv_add_cart = findViewById(R.id.tv_add_cart);
        wishlist_menu = findViewById(R.id.toolbar_menu_wishlist);
        cart_badge = findViewById(R.id.cart_badge);
        // on swipe
        swipe_refresh.setOnRefreshListener(() -> requestAction());

        lyt_add_cart.setOnClickListener(view -> {
            if (product == null || (product.name != null && product.name.equals(""))) {
                Toast.makeText(getApplicationContext(), R.string.please_wait_text, Toast.LENGTH_SHORT).show();
                return;
            }
            toggleCartButton();
        });

        (findViewById(R.id.toolbar_menu_back)).setOnClickListener(v -> onBackAction());

        (findViewById(R.id.toolbar_menu_wishlist)).setOnClickListener(v -> {
            if (product == null) return;
            if (product.name == null || product.name.equals("")) {
                Toast.makeText(this, R.string.cannot_add_wishlist, Toast.LENGTH_SHORT).show();
                return;
            }
            if (flag_wishlist) {
                db.deleteWishlist(product_id);
                Toast.makeText(this, R.string.remove_wishlist, Toast.LENGTH_SHORT).show();
            } else {
                Wishlist w = new Wishlist(product.id, product.name, product.image, System.currentTimeMillis());
                db.saveWishlist(w);
                Toast.makeText(this, R.string.add_wishlist, Toast.LENGTH_SHORT).show();
            }
            refreshWishlistMenu();
        });

        (findViewById(R.id.toolbar_menu_cart)).setOnClickListener(v -> ActivityShoppingCart.navigate(this));

        refreshWishlistMenu();
    }

    private void requestAction() {
        showFailedView(false, "");
        swipeProgress(true);
        new Handler(getMainLooper()).postDelayed(() -> requestNewsInfoDetailsApi(), 1000);
    }

    private void onFailRequest() {
        swipeProgress(false);
        if (NetworkCheck.isConnect(this)) {
            showFailedView(true, getString(R.string.failed_text));
        } else {
            showFailedView(true, getString(R.string.no_internet_text));
        }
    }


    private void prepareAds() {
        AdNetworkHelper adNetworkHelper = new AdNetworkHelper(this);
        adNetworkHelper.loadBannerAd(AppConfig.ads.ad_product_details);
    }

    private void requestNewsInfoDetailsApi() {
        API api = RestAdapter.createAPI();
        callbackCall = api.getProductDetails(product_id);
        callbackCall.enqueue(new Callback<RespProductDetails>() {
            @Override
            public void onResponse(Call<RespProductDetails> call, Response<RespProductDetails> response) {
                RespProductDetails resp = response.body();
                if (resp != null && resp.status.equals("success")) {
                    product = resp.product;
                    displayPostData();
                    swipeProgress(false);
                } else {
                    onFailRequest();
                }
            }

            @Override
            public void onFailure(Call<RespProductDetails> call, Throwable t) {
                Log.e("onFailure", t.getMessage());
                if (!call.isCanceled()) onFailRequest();
            }
        });
    }

    private void displayPostData() {
        ((TextView) findViewById(R.id.title)).setText(Html.fromHtml(product.name));

        webview = (WebView) findViewById(R.id.content);
        Tools.displayContentHtml(this, webview, product.description);

        ((TextView) findViewById(R.id.date)).setText(Tools.getFormattedDate(product.last_update));

        TextView price = (TextView) findViewById(R.id.price);
        TextView price_strike = (TextView) findViewById(R.id.price_strike);

        // handle discount view
        if (product.price_discount > 0) {
            price.setText(Tools.getFormattedPrice(product.price_discount, this));
            price_strike.setText(Tools.getFormattedPrice(product.price, this));
            price_strike.setPaintFlags(price_strike.getPaintFlags() | Paint.STRIKE_THRU_TEXT_FLAG);
            price_strike.setVisibility(View.VISIBLE);
        } else {
            price.setText(Tools.getFormattedPrice(product.price, this));
            price_strike.setVisibility(View.GONE);
        }

        if (product.status.equalsIgnoreCase("READY STOCK")) {
            ((TextView) findViewById(R.id.status)).setText(getString(R.string.ready_stock));
        } else if (product.status.equalsIgnoreCase("OUT OF STOCK")) {
            ((TextView) findViewById(R.id.status)).setText(getString(R.string.out_of_stock));
        } else if (product.status.equalsIgnoreCase("SUSPEND")) {
            ((TextView) findViewById(R.id.status)).setText(getString(R.string.suspend));
        } else {
            ((TextView) findViewById(R.id.status)).setText(product.status);
        }

        // display Image slider
        displayImageSlider();

        // display category list at bottom
        displayCategoryProduct();

        Toast.makeText(this, R.string.msg_data_loaded, Toast.LENGTH_SHORT).show();

    }

    private void displayImageSlider() {
        final LinearLayout layout_dots = (LinearLayout) findViewById(R.id.layout_dots);
        ViewPager viewPager = (ViewPager) findViewById(R.id.pager);
        final AdapterProductImage adapterSlider = new AdapterProductImage(this, new ArrayList<ProductImage>());

        final List<ProductImage> productImages = new ArrayList<>();
        ProductImage p = new ProductImage();
        p.product_id = product.id;
        p.name = product.image;
        productImages.add(p);
        if (product.product_images != null) productImages.addAll(product.product_images);
        adapterSlider.setItems(productImages);
        viewPager.setAdapter(adapterSlider);

        // displaying selected image first
        viewPager.setCurrentItem(0);
        addBottomDots(layout_dots, adapterSlider.getCount(), 0);
        viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int pos, float positionOffset, int positionOffsetPixels) {
            }

            @Override
            public void onPageSelected(int pos) {
                addBottomDots(layout_dots, adapterSlider.getCount(), pos);
            }

            @Override
            public void onPageScrollStateChanged(int state) {
            }
        });


        final ArrayList<String> images_list = new ArrayList<>();
        for (ProductImage img : productImages) {
            images_list.add(Constant.getURLimgProduct(img.name));
        }

        adapterSlider.setOnItemClickListener((view, obj, pos) -> {
            ActivityFullScreenImage.navigate(this, images_list, pos, view);
        });
    }

    private void displayCategoryProduct() {
        FlexboxLayout flexCategory = findViewById(R.id.flex_category);
        if (product.categories == null || product.categories.size() == 0) {
            flexCategory.setVisibility(View.GONE);
            return;
        }
        flexCategory.removeAllViews();
        for (Category cat : product.categories) {
            TextView textView = new TextView(this);
            // set margin
            int marginHorizontal = getResources().getDimensionPixelOffset(R.dimen.spacing_4);
            int marginVertical = getResources().getDimensionPixelOffset(R.dimen.spacing_4);
            LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);
            params.setMargins(marginHorizontal, marginVertical, marginHorizontal, marginVertical);
            textView.setLayoutParams(params);

            // set padding
            int paddingHorizontal = getResources().getDimensionPixelOffset(R.dimen.spacing_15);
            int paddingVertical = textView.getPaddingTop();
            textView.setClickable(true);
            textView.setFocusable(true);
            textView.setPadding(paddingHorizontal, paddingVertical, paddingHorizontal, paddingVertical);
            textView.setGravity(Gravity.CENTER);

            // set font and color
            textView.setBackgroundResource(R.drawable.button_category);
            textView.setText(cat.name);
            textView.setTextColor(getResources().getColor(R.color.textIconPrimary));
            textView.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(R.dimen.txt_small));

            flexCategory.addView(textView);
        }

    }

    private void addBottomDots(LinearLayout layout_dots, int size, int current) {
        ImageView[] dots = new ImageView[size];

        layout_dots.removeAllViews();
        for (int i = 0; i < dots.length; i++) {
            dots[i] = new ImageView(this);
            int width_height = 15;
            LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(new ViewGroup.LayoutParams(width_height, width_height));
            params.setMargins(10, 10, 10, 10);
            dots[i].setLayoutParams(params);
            dots[i].setImageResource(R.drawable.shape_circle);
            dots[i].setColorFilter(ContextCompat.getColor(this, R.color.dark_overlay));
            layout_dots.addView(dots[i]);
        }

        if (dots.length > 0)
            dots[current].setColorFilter(ContextCompat.getColor(this, R.color.primary));
    }

    private void showFailedView(boolean show, String message) {
        View lyt_failed = (View) findViewById(R.id.lyt_failed);
        View lyt_main_content = (View) findViewById(R.id.nested_content);

        ((TextView) findViewById(R.id.failed_message)).setText(message);
        if (show) {
            lyt_main_content.setVisibility(View.GONE);
            lyt_failed.setVisibility(View.VISIBLE);
        } else {
            lyt_main_content.setVisibility(View.VISIBLE);
            lyt_failed.setVisibility(View.GONE);
        }
        ((Button) findViewById(R.id.failed_retry)).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                requestAction();
            }
        });
    }

    private void swipeProgress(final boolean show) {
        if (!show) {
            swipe_refresh.setRefreshing(show);
            return;
        }
        swipe_refresh.post(new Runnable() {
            @Override
            public void run() {
                swipe_refresh.setRefreshing(show);
            }
        });
    }

    @Override
    public void onBackPressed() {
        onBackAction();
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (webview != null) webview.onPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (webview != null) webview.onResume();
        refreshCartButton();
        refreshCartBadge();
    }

    private void onBackAction() {
        if (from_notif) {
            if (ActivityMain.active) {
                finish();
            } else {
                Intent intent = new Intent(getApplicationContext(), ActivitySplash.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                startActivity(intent);
                finish();
            }
        } else {
            super.onBackPressed();
        }
    }

    private void refreshWishlistMenu() {
        Wishlist w = db.getWishlist(product_id);
        flag_wishlist = (w != null);
        if (flag_wishlist) {
            wishlist_menu.setImageResource(R.drawable.ic_wish);
        } else {
            wishlist_menu.setImageResource(R.drawable.ic_wish_outline);
        }
    }

    private void toggleCartButton() {
        if (flag_cart) {
            db.deleteActiveCart(product_id);
            Toast.makeText(this, R.string.remove_cart, Toast.LENGTH_SHORT).show();
        } else {
            // check stock product
            if (product.stock == 0 || product.status.equalsIgnoreCase("OUT OF STOCK")) {
                Toast.makeText(this, R.string.msg_out_of_stock, Toast.LENGTH_SHORT).show();
                return;
            }
            if (product.status.equalsIgnoreCase("SUSPEND")) {
                Toast.makeText(this, R.string.msg_suspend, Toast.LENGTH_SHORT).show();
                return;
            }
            Double selected_price = product.price_discount > 0 ? product.price_discount : product.price;
            Cart cart = new Cart(product.id, product.name, product.image, 1, product.stock, selected_price, System.currentTimeMillis());
            db.saveCart(cart);
            Toast.makeText(this, R.string.add_cart, Toast.LENGTH_SHORT).show();
        }
        refreshCartButton();
        refreshCartBadge();
    }

    private void refreshCartButton() {
        Cart c = db.getCart(product_id);
        flag_cart = (c != null);
        if (flag_cart) {
            lyt_add_cart.setBackgroundColor(getResources().getColor(R.color.colorRemoveCart));
            tv_add_cart.setText(R.string.bt_remove_cart);
        } else {
            lyt_add_cart.setBackgroundColor(getResources().getColor(R.color.colorAddCart));
            tv_add_cart.setText(R.string.bt_add_cart);
        }
    }

    private void refreshCartBadge() {
        int new_cart_count = db.getActiveCartSize();
        if (new_cart_count != cart_count) {
            cart_count = new_cart_count;
            setupBadge();
        }
    }

    private void setupBadge() {
        if (cart_count == 0) {
            cart_badge.setVisibility(View.GONE);
        } else {
            cart_badge.setVisibility(View.VISIBLE);
            String count_txt = cart_count + "";
            if (cart_count > 9) count_txt = "9+";
            ((TextView) cart_badge.findViewById(R.id.counter)).setText(count_txt);
        }
    }
}
