package com.app.markeet.activity;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.app.markeet.AppConfig;
import com.app.markeet.R;
import com.app.markeet.adapter.AdapterWishlist;
import com.app.markeet.data.DatabaseHandler;
import com.app.markeet.model.Wishlist;
import com.app.markeet.utils.Tools;
import com.google.android.material.snackbar.Snackbar;

import java.util.ArrayList;
import java.util.List;

public class ActivityWishlist extends AppCompatActivity {

    public static void navigate(Activity activity) {
        Intent i = new Intent(activity, ActivityWishlist.class);
        activity.startActivity(i);
    }

    private View parent_view;
    private RecyclerView recyclerView;
    private DatabaseHandler db;
    private AdapterWishlist adapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_wishlist);
        db = new DatabaseHandler(this);

        iniComponent();
    }

    private void iniComponent() {
        parent_view = findViewById(android.R.id.content);
        (findViewById(R.id.toolbar_menu_back)).setOnClickListener(v -> onBackPressed());
        (findViewById(R.id.toolbar_menu_delete)).setOnClickListener(v -> {
            if (adapter.getItemCount() == 0) {
                Snackbar.make(parent_view, R.string.msg_wishlist_empty, Snackbar.LENGTH_SHORT).show();
                return;
            }
            dialogDeleteConfirmation();
        });

        recyclerView = (RecyclerView) findViewById(R.id.recyclerView);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        //set data and list adapter
        adapter = new AdapterWishlist(this, recyclerView, new ArrayList<Wishlist>());
        recyclerView.setAdapter(adapter);
        recyclerView.setNestedScrollingEnabled(false);

        adapter.setOnItemClickListener((view, obj, pos) -> ActivityProductDetails.navigate(ActivityWishlist.this, obj.product_id, false));

        startLoadMoreAdapter();
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
    }

    private void startLoadMoreAdapter() {
        adapter.resetListData();
        List<Wishlist> items = db.getWishlistByPage(AppConfig.general.wishlist_page, 0);
        adapter.insertData(items);
        showNoItemView();
        final int item_count = (int) db.getWishlistSize();
        // detect when scroll reach bottom
        adapter.setOnLoadMoreListener(current_page -> {
            if (item_count > adapter.getItemCount() && current_page != 0) {
                displayDataByPage(current_page);
            } else {
                adapter.setLoaded();
            }
        });
    }

    private void displayDataByPage(final int next_page) {
        adapter.setLoading();
        new Handler(getMainLooper()).postDelayed(() -> {
            List<Wishlist> items = db.getWishlistByPage(AppConfig.general.wishlist_page, (next_page * AppConfig.general.wishlist_page));
            adapter.insertData(items);
            showNoItemView();
        }, 500);
    }

    private void showNoItemView() {
        View lyt_no_item = (View) findViewById(R.id.lyt_no_item);
        if (adapter.getItemCount() == 0) {
            lyt_no_item.setVisibility(View.VISIBLE);
        } else {
            lyt_no_item.setVisibility(View.GONE);
        }
    }


    public void dialogDeleteConfirmation() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.title_delete_confirm);
        builder.setMessage(getString(R.string.content_delete_confirm) + getString(R.string.title_activity_wishlist));
        builder.setPositiveButton(R.string.YES, (di, i) -> {
            di.dismiss();
            db.deleteWishlist();
            startLoadMoreAdapter();
            Snackbar.make(parent_view, R.string.delete_success, Snackbar.LENGTH_SHORT).show();
        });
        builder.setNegativeButton(R.string.CANCEL, null);
        builder.show();
    }
}
