package com.app.markeet.adapter;

import android.content.Context;
import android.graphics.Color;
import android.os.Build;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.core.text.HtmlCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.app.markeet.AppConfig;
import com.app.markeet.R;
import com.app.markeet.data.Constant;
import com.app.markeet.databinding.ItemCategoryBinding;
import com.app.markeet.databinding.ItemLoadingBinding;
import com.app.markeet.model.Category;
import com.app.markeet.utils.Tools;

import java.util.ArrayList;
import java.util.List;


public class AdapterCategory extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private Context ctx;
    private List<Category> items = new ArrayList<>();

    private final int VIEW_ITEM = 2;
    private final int VIEW_PROG = 0;
    private boolean loading = true;
    private String status;

    private AdapterListener<Category> listener;

    public void setListener(AdapterListener<Category> listener) {
        this.listener = listener;
    }

    // Provide a suitable constructor (depends on the kind of dataset)
    public AdapterCategory(Context context) {
        ctx = context;
    }

    public class OriginalViewHolder extends RecyclerView.ViewHolder {
        ItemCategoryBinding binding;

        public OriginalViewHolder(@NonNull ItemCategoryBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }
    }

    public static class ProgressViewHolder extends RecyclerView.ViewHolder {
        ItemLoadingBinding binding;

        public ProgressViewHolder(@NonNull ItemLoadingBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }
    }

    public void insertData(List<Category> items) {
        setLoaded();
        int positionStart = getItemCount();
        int itemCount = items.size();
        this.items.addAll(items);
        notifyItemRangeInserted(positionStart, itemCount);
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder vh;
        if (viewType == VIEW_ITEM) {
            ItemCategoryBinding v = ItemCategoryBinding.inflate(LayoutInflater.from(parent.getContext()), parent, false);
            vh = new OriginalViewHolder(v);
        } else {
            ItemLoadingBinding v = ItemLoadingBinding.inflate(LayoutInflater.from(parent.getContext()), parent, false);
            vh = new AdapterHome.ProgressViewHolder(v);
        }
        return vh;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position, @NonNull List<Object> payloads) {
        super.onBindViewHolder(holder, position, payloads);

    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int pos) {
        int position = holder.getBindingAdapterPosition();
        if (holder instanceof OriginalViewHolder) {
            final Category c = items.get(position);
            ItemCategoryBinding v = ((OriginalViewHolder) holder).binding;
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                v.name.setText(Html.fromHtml(c.name, Html.FROM_HTML_MODE_COMPACT));
            } else {
                v.name.setText(HtmlCompat.fromHtml(c.name, HtmlCompat.FROM_HTML_MODE_LEGACY));
            }
            //vItem.name.setText(c.name);
            v.brief.setText(c.brief);
            v.lytColor.setBackgroundColor(Color.parseColor(c.color));
            Tools.displayImageThumbnail(ctx, v.image, Constant.getURLimgCategory(c.icon), 0.5f);

            if (AppConfig.general.tint_category_icon) {
                v.image.setColorFilter(ctx.getResources().getColor(R.color.textIconPrimary));
            }

            v.name.setText(c.name);
            v.lytParent.setOnClickListener(view -> {
                if (listener == null) return;
                listener.onClick(view, null, c, position);
            });
        } else {
            ItemLoadingBinding v = ((ProgressViewHolder) holder).binding;
            v.progressLoading.setVisibility(status == null ? View.VISIBLE : View.INVISIBLE);
            v.statusLoading.setVisibility(status == null ? View.INVISIBLE : View.VISIBLE);

            if (status == null) return;
            v.statusLoading.setText(status);
            v.statusLoading.setOnClickListener(view -> {
                setLoaded();
                onLoadMore();
            });
        }


    }

    // Return the size of your dataset (invoked by the layout manager)
    @Override
    public int getItemCount() {
        return items.size();
    }

    @Override
    public int getItemViewType(int position) {
        return this.items.get(position) != null ? VIEW_ITEM : VIEW_PROG;
    }

    public void setLoaded() {
        status = null;
        loading = false;
        int last_index = getItemCount() - 1;
        if (last_index > -1 && items.get(last_index) == null) {
            items.remove(last_index);
            notifyItemRemoved(last_index);
        }
    }

    public void setLoadingOrFailed(String status) {
        setLoaded();
        this.status = status;
        this.items.add(null);
        notifyItemInserted(getItemCount() - 1);
        loading = true;
    }

    public void resetListData() {
        this.items = new ArrayList<>();
        notifyDataSetChanged();
    }

    private void onLoadMore() {
        listener.onLoadMore(1);
        loading = true;
        status = null;
    }

}