package com.app.markeet.adapter;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PorterDuff;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.viewpager.widget.ViewPager;

import com.app.markeet.R;
import com.app.markeet.connection.callbacks.RespFeaturedNews;
import com.app.markeet.data.Constant;
import com.app.markeet.databinding.ItemLoadingBinding;
import com.app.markeet.databinding.ItemProductBinding;
import com.app.markeet.databinding.ItemSectionSliderBinding;
import com.app.markeet.model.NewsInfo;
import com.app.markeet.model.Product;
import com.app.markeet.utils.Tools;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class AdapterHome extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    public enum ActionType {
        ITEM, SLIDER,
    }

    private final Context ctx;
    private List<Object> items = new ArrayList<>();
    private List<Object> items_temp = new ArrayList<>();

    private final int VIEW_ITEM = 200;
    private final int VIEW_SLIDER = 400;
    private final int VIEW_PROG = 0;
    private boolean loading = true;
    private String status;
    private int page = 0;

    private AdapterListener<Object> listener;

    public void setListener(AdapterListener<Object> listener) {
        this.listener = listener;
    }

    // Provide a suitable constructor (depends on the kind of dataset)
    public AdapterHome(Context context, RecyclerView view, int page) {
        this.page = page;
        ctx = context;
        lastItemViewDetector(view);
    }

    public class OriginalViewHolder extends RecyclerView.ViewHolder {
        ItemProductBinding binding;

        public OriginalViewHolder(@NonNull ItemProductBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }
    }

    public class SectionSliderViewHolder extends RecyclerView.ViewHolder {
        ItemSectionSliderBinding binding;

        public SectionSliderViewHolder(@NonNull ItemSectionSliderBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }
    }

    public static class ProgressViewHolder extends RecyclerView.ViewHolder {
        ItemLoadingBinding binding;

        public ProgressViewHolder(@NonNull ItemLoadingBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder vh;

        if (viewType == VIEW_ITEM) {
            ItemProductBinding v = ItemProductBinding.inflate(LayoutInflater.from(parent.getContext()), parent, false);
            vh = new OriginalViewHolder(v);
        } else if (viewType == VIEW_SLIDER) {
            ItemSectionSliderBinding v = ItemSectionSliderBinding.inflate(LayoutInflater.from(parent.getContext()), parent, false);
            vh = new SectionSliderViewHolder(v);
        } else {
            ItemLoadingBinding v = ItemLoadingBinding.inflate(LayoutInflater.from(parent.getContext()), parent, false);
            vh = new ProgressViewHolder(v);
        }
        return vh;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position, @NonNull List<Object> payloads) {
        super.onBindViewHolder(holder, position, payloads);

    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int pos) {
        int position = holder.getBindingAdapterPosition();
        Object obj = items.get(position);
        if (holder instanceof OriginalViewHolder) {
            ItemProductBinding v = ((OriginalViewHolder) holder).binding;
            final Product p = (Product) obj;
            v.name.setText(p.name);

            // handle discount view
            if (p.price_discount > 0) {
                v.price.setText(Tools.getFormattedPrice(p.price_discount, ctx));
                v.priceStrike.setText(Tools.getFormattedPrice(p.price, ctx));
                v.priceStrike.setPaintFlags(v.priceStrike.getPaintFlags() | Paint.STRIKE_THRU_TEXT_FLAG);
                v.priceStrike.setVisibility(View.VISIBLE);
                v.saleBadge.setVisibility(View.VISIBLE);
            } else {
                v.price.setText(Tools.getFormattedPrice(p.price, ctx));
                v.priceStrike.setVisibility(View.GONE);
                v.saleBadge.setVisibility(View.GONE);
            }
            Tools.displayImageOriginal(ctx, v.image, Constant.getURLimgProduct(p.image));
            v.lytParent.setOnClickListener(view -> {
                if (listener == null) return;
                listener.onClick(view, ActionType.ITEM.name(), p, position);
            });

        } else if (holder instanceof SectionSliderViewHolder) {
            RespFeaturedNews sliderList = (RespFeaturedNews) obj;
            final ItemSectionSliderBinding v = ((SectionSliderViewHolder) holder).binding;
            final AdapterFeaturedNews adapter = new AdapterFeaturedNews(ctx, new ArrayList<NewsInfo>());

            adapter.setItems(sliderList.news_infos);
            v.viewPager.setAdapter(adapter);

            ViewGroup.LayoutParams params = v.viewPager.getLayoutParams();
            params.height = Tools.getFeaturedNewsImageHeight(ctx);
            v.viewPager.setLayoutParams(params);

            // displaying selected image first
            v.viewPager.setCurrentItem(0);
            //v.title.setText(adapter.getItem(0).title);
            addBottomDots(v.layoutDots, adapter.getCount(), 0);
            v.viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
                @Override
                public void onPageScrolled(int pos, float positionOffset, int positionOffsetPixels) {
                }

                @Override
                public void onPageSelected(int pos) {
                    NewsInfo cur = adapter.getItem(pos);
                    //v.title.setText(cur.title);
                    addBottomDots(v.layoutDots, adapter.getCount(), pos);
                }

                @Override
                public void onPageScrollStateChanged(int state) {
                }
            });

            startAutoSlider(v.viewPager, adapter.getCount());

            adapter.setOnItemClickListener((view, obj1) -> {
                if (listener == null) return;
                listener.onClick(view, ActionType.SLIDER.name(), obj1, position);
            });

        } else {

            final ItemLoadingBinding v = ((ProgressViewHolder) holder).binding;
            v.progressLoading.setVisibility(status == null ? View.VISIBLE : View.GONE);
            v.statusLoading.setVisibility(status == null ? View.GONE : View.VISIBLE);

            if (status == null) return;
            v.statusLoading.setText(status);
            v.statusLoading.setOnClickListener(view -> {
                setLoaded();
                onLoadMore();
            });
        }
    }

    @Override
    public void onViewDetachedFromWindow(@NonNull RecyclerView.ViewHolder holder) {
        super.onViewDetachedFromWindow(holder);
        if (runnable != null) handler.removeCallbacks(runnable);
    }

    // Return the size of your dataset (invoked by the layout manager)
    @Override
    public int getItemCount() {
        return items.size();
    }

    @Override
    public int getItemViewType(int position) {
        Object obj = items.get(position);
        if (obj instanceof Product) {
            return VIEW_ITEM;
        } else if (obj instanceof RespFeaturedNews) {
            return VIEW_SLIDER;
        } else {
            return VIEW_PROG;
        }
    }

    private Runnable runnable = null;
    private Handler handler = new Handler();

    private void startAutoSlider(final ViewPager viewPager, final int count) {
        //if(runnable != null) return;
        runnable = () -> {
            int pos = viewPager.getCurrentItem();
            pos = pos + 1;
            if (pos >= count) pos = 0;
            viewPager.setCurrentItem(pos);
            handler.postDelayed(runnable, 3000);
        };
        handler.postDelayed(runnable, 3000);
    }

    private void addBottomDots(LinearLayout layout_dots, int size, int current) {
        ImageView[] dots = new ImageView[size];
        layout_dots.removeAllViews();
        for (int i = 0; i < dots.length; i++) {
            dots[i] = new ImageView(ctx);
            int width_height = ctx.getResources().getDimensionPixelOffset(R.dimen.spacing_6);
            LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(new ViewGroup.LayoutParams(width_height, width_height));
            params.setMargins(6, 0, 6, 0);
            dots[i].setLayoutParams(params);
            dots[i].setImageResource(R.drawable.shape_circle_outline);
            dots[i].setColorFilter(Color.WHITE, PorterDuff.Mode.SRC_ATOP);
            layout_dots.addView(dots[i]);
        }

        if (dots.length > 0) {
            dots[current].setImageResource(R.drawable.shape_circle);
            dots[current].setColorFilter(Color.WHITE, PorterDuff.Mode.SRC_ATOP);
        }
    }

    public void insertData(List<Product> items) {
        setLoaded();
        this.items.addAll(items);
    }

    public void setLoaded() {
        status = null;
        loading = false;
        items.removeAll(Collections.singleton(null));
        notifyDataSetChanged();
    }

    public void clearListProduct() {
        int listing_pos = -1;
        for (int i = 0; i < this.items.size(); i++) {
            if (this.items.get(i) instanceof Product) {
                listing_pos = i;
                break;
            }
        }
        if (listing_pos == -1) return;
        int count = items.size();
        items.subList(listing_pos, count).clear();
        notifyItemRangeRemoved(listing_pos, count);
    }

    public void setLoadingOrFailed(String status) {
        setLoaded();
        this.status = status;
        this.items.add(null);
        notifyItemInserted(getItemCount() - 1);
        loading = true;
    }

    public void resetListData() {
        this.items = new ArrayList<>();
        notifyDataSetChanged();
    }

    public void setSlider(List<NewsInfo> items) {
        RespFeaturedNews r = new RespFeaturedNews();
        r.news_infos = items;
        this.items.add(r);
        notifyDataSetChanged();
    }

    boolean scrollDown = false;

    private void lastItemViewDetector(RecyclerView recyclerView) {
        if (recyclerView.getLayoutManager() instanceof GridLayoutManager) {
            final GridLayoutManager layoutManager = (GridLayoutManager) recyclerView.getLayoutManager();
            recyclerView.addOnScrollListener(new RecyclerView.OnScrollListener() {
                @Override
                public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                    super.onScrollStateChanged(recyclerView, newState);
                    if (newState == RecyclerView.SCROLL_STATE_IDLE || !scrollDown) return;
                    int lastPos = layoutManager.findLastVisibleItemPosition();
                    boolean bottom = lastPos >= getItemCount() - page;
                    if (!loading && bottom && listener != null) {
                        onLoadMore();
                    }
                }

                @Override
                public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                    super.onScrolled(recyclerView, dx, dy);
                    scrollDown = dy > 0;
                }
            });

            layoutManager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
                @Override
                public int getSpanSize(int position) {
                    int type = getItemViewType(position);
                    int spanCount = layoutManager.getSpanCount();
                    return type == VIEW_ITEM ? 1 : spanCount;
                }
            });
        }
    }

    private void onLoadMore() {
        int current_page = getItemCount() / page;
        listener.onLoadMore(current_page);
        loading = true;
        status = null;
    }

}
