package com.app.markeet.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.app.markeet.data.Constant;
import com.app.markeet.databinding.ItemLoadingBinding;
import com.app.markeet.databinding.ItemNewsInfoBinding;
import com.app.markeet.model.NewsInfo;
import com.app.markeet.utils.Tools;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class AdapterNews extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private final Context ctx;
    private List<NewsInfo> items = new ArrayList<>();

    private final int VIEW_ITEM = 200;
    private final int VIEW_PROG = 0;
    private boolean loading = true;
    private String status;
    private int page = 0;

    private AdapterListener<NewsInfo> listener;

    public void setListener(AdapterListener<NewsInfo> listener) {
        this.listener = listener;
    }

    // Provide a suitable constructor (depends on the kind of dataset)
    public AdapterNews(Context context, RecyclerView view, int page) {
        this.page = page;
        ctx = context;
        lastItemViewDetector(view);
    }

    public class OriginalViewHolder extends RecyclerView.ViewHolder {
        ItemNewsInfoBinding binding;

        public OriginalViewHolder(@NonNull ItemNewsInfoBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }
    }

    public static class ProgressViewHolder extends RecyclerView.ViewHolder {
        ItemLoadingBinding binding;

        public ProgressViewHolder(@NonNull ItemLoadingBinding binding) {
            super(binding.getRoot());
            this.binding = binding;
        }
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder vh;

        if (viewType == VIEW_ITEM) {
            ItemNewsInfoBinding v = ItemNewsInfoBinding.inflate(LayoutInflater.from(parent.getContext()), parent, false);
            vh = new OriginalViewHolder(v);
        } else {
            ItemLoadingBinding v = ItemLoadingBinding.inflate(LayoutInflater.from(parent.getContext()), parent, false);
            vh = new ProgressViewHolder(v);
        }
        return vh;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int pos) {
        int position = holder.getBindingAdapterPosition();
        NewsInfo o = items.get(position);
        if (holder instanceof OriginalViewHolder) {
            ItemNewsInfoBinding v = ((OriginalViewHolder) holder).binding;

            v.title.setText(o.title);
            v.briefContent.setText(o.brief_content);
            v.date.setText(Tools.getFormattedDateSimple(o.last_update));
            if (o.status.equalsIgnoreCase("FEATURED")) {
                v.featured.setVisibility(View.VISIBLE);
            } else {
                v.featured.setVisibility(View.GONE);
            }
            Tools.displayImageOriginal(ctx, v.image, Constant.getURLimgNews(o.image));
            v.lytParent.setOnClickListener(view -> {
                if (listener == null) return;
                listener.onClick(view, null, o, position);
            });

        } else {

            final ProgressViewHolder v = (ProgressViewHolder) holder;
            v.binding.progressLoading.setVisibility(status == null ? View.VISIBLE : View.GONE);
            v.binding.statusLoading.setVisibility(status == null ? View.GONE : View.VISIBLE);

            if (status == null) return;
            v.binding.statusLoading.setText(status);
            v.binding.statusLoading.setOnClickListener(view -> {
                setLoaded();
                onLoadMore();
            });
        }
    }

    // Return the size of your dataset (invoked by the layout manager)
    @Override
    public int getItemCount() {
        return items.size();
    }

    @Override
    public int getItemViewType(int position) {
        NewsInfo obj = items.get(position);
        if (obj instanceof NewsInfo) {
            return VIEW_ITEM;
        } else {
            return VIEW_PROG;
        }
    }

    public void insertData(List<NewsInfo> items) {
        setLoaded();
        this.items.addAll(items);
    }

    public void setLoaded() {
        status = null;
        loading = false;
        items.removeAll(Collections.singleton(null));
        notifyDataSetChanged();
    }

    public void resetListData() {
        this.items = new ArrayList<>();
        notifyDataSetChanged();
    }

    public void setLoadingOrFailed(String status) {
        setLoaded();
        this.status = status;
        this.items.add(null);
        notifyItemInserted(getItemCount() - 1);
        loading = true;
    }

    boolean scrollDown = false;

    private void lastItemViewDetector(RecyclerView recyclerView) {
        if (recyclerView.getLayoutManager() instanceof LinearLayoutManager) {
            final LinearLayoutManager layoutManager = (LinearLayoutManager) recyclerView.getLayoutManager();
            recyclerView.addOnScrollListener(new RecyclerView.OnScrollListener() {
                @Override
                public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                    super.onScrollStateChanged(recyclerView, newState);
                    if (newState == RecyclerView.SCROLL_STATE_IDLE || !scrollDown) return;
                    int lastPos = layoutManager.findLastVisibleItemPosition();
                    boolean bottom = lastPos >= getItemCount() - page;
                    if (!loading && bottom && listener != null) {
                        onLoadMore();
                    }
                }

                @Override
                public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                    super.onScrolled(recyclerView, dx, dy);
                    scrollDown = dy > 0;
                }
            });
        }
    }

    private void onLoadMore() {
        int current_page = getItemCount() / page;
        listener.onLoadMore(current_page);
        loading = true;
        status = null;
    }

}
