package com.app.markeet.data;

import android.app.Application;
import android.content.Context;
import android.os.Handler;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.multidex.MultiDex;

import com.app.markeet.AppConfig;
import com.app.markeet.BuildConfig;
import com.app.markeet.R;
import com.app.markeet.connection.API;
import com.app.markeet.connection.RestAdapter;
import com.app.markeet.connection.callbacks.RespDevice;
import com.app.markeet.model.DeviceInfo;
import com.app.markeet.model.SortBy;
import com.app.markeet.utils.NetworkCheck;
import com.app.markeet.utils.Tools;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.FirebaseApp;
import com.google.firebase.analytics.FirebaseAnalytics;
import com.google.firebase.messaging.FirebaseMessaging;
import com.google.firebase.remoteconfig.FirebaseRemoteConfig;
import com.google.firebase.remoteconfig.FirebaseRemoteConfigSettings;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class ThisApp extends Application {

    private static ThisApp mInstance;
    private static SharedPref sharedPref;
    private FirebaseRemoteConfig firebaseRemoteConfig;

    public static synchronized SharedPref pref() {
        return sharedPref;
    }

    public static synchronized ThisApp get() {
        return mInstance;
    }

    private int fcm_count = 0;
    private final int FCM_MAX_COUNT = 10;
    private List<SortBy> sorts = new ArrayList<>();


    @Override
    protected void attachBaseContext(Context base) {
        super.attachBaseContext(base);
        MultiDex.install(this);
    }

    @Override
    public void onCreate() {
        super.onCreate();
        mInstance = this;
        sharedPref = new SharedPref(this);

        // initialize firebase
        initFirebase();
        initRemoteConfig();

        initSortByData();
    }

    public void obtainAndSubmitToServer() {
        // obtain regId & registering device to server
        obtainFirebaseToken();
        subscribeTopicNotif();
    }

    private void obtainFirebaseToken() {
        if (NetworkCheck.isConnect(this)) {
            fcm_count++;

            FirebaseMessaging.getInstance().getToken().addOnCompleteListener(task -> {
                if (!task.isSuccessful()) {
                    if (fcm_count > FCM_MAX_COUNT) return;
                    new Handler().postDelayed(() -> obtainFirebaseToken(), 500);
                    return;
                }

                // Get new FCM registration token
                String regId = task.getResult();
                sharedPref.setFcmRegId(regId);
                if (!TextUtils.isEmpty(regId)) sendRegistrationToServer(regId);
            });

        }
    }

    private void sendRegistrationToServer(String token) {
        Log.d("FCM_TOKEN", token + "");
        DeviceInfo deviceInfo = Tools.getDeviceInfo(this);
        deviceInfo.regid = token;

        API api = RestAdapter.createAPI();
        Call<RespDevice> callbackCall = api.registerDevice(AppConfig.general.security_code, deviceInfo);
        callbackCall.enqueue(new Callback<RespDevice>() {
            @Override
            public void onResponse(Call<RespDevice> call, Response<RespDevice> response) {
                RespDevice resp = response.body();
                if (resp != null && resp.status.equals("success")) {

                }
            }

            @Override
            public void onFailure(Call<RespDevice> call, Throwable t) {
                Log.e("onFailure", t.getMessage());
            }
        });
    }

    private void subscribeTopicNotif() {
        if (sharedPref.isSubscibeNotif()) return;
        FirebaseMessaging.getInstance().subscribeToTopic("ALL-DEVICE").addOnCompleteListener(new OnCompleteListener<Void>() {
            @Override
            public void onComplete(@NonNull Task<Void> task) {
                sharedPref.setSubscibeNotif(task.isSuccessful());
            }
        });
    }

    private void initSortByData() {
        sorts.add(new SortBy(getString(R.string.sort_new_old), "id", "DESC"));
        sorts.add(new SortBy(getString(R.string.sort_old_new), "id", "ASC"));
        sorts.add(new SortBy(getString(R.string.sort_low_high), "price", "ASC"));
        sorts.add(new SortBy(getString(R.string.sort_high_low), "price", "DESC"));
        sorts.add(new SortBy(getString(R.string.sort_discount), "price_discount", "DESC"));
    }

    public List<SortBy> getSorts() {
        return sorts;
    }

    private void initRemoteConfig() {
        firebaseRemoteConfig = FirebaseRemoteConfig.getInstance();
        if (!AppConfig.USE_REMOTE_CONFIG) return;
        FirebaseRemoteConfigSettings configSettings = new FirebaseRemoteConfigSettings.Builder()
                .setMinimumFetchIntervalInSeconds(BuildConfig.DEBUG ? 0 : 60)
                .setFetchTimeoutInSeconds(4)
                .build();
        firebaseRemoteConfig.setConfigSettingsAsync(configSettings);
    }

    private void initFirebase() {
        // Obtain the Firebase Analytics.
        FirebaseApp.initializeApp(this);
        FirebaseAnalytics.getInstance(this);
    }

    public FirebaseRemoteConfig getFirebaseRemoteConfig() {
        return firebaseRemoteConfig;
    }
}
