package com.app.markeet.fragment;

import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.GridLayoutManager;

import com.app.markeet.AppConfig;
import com.app.markeet.R;
import com.app.markeet.activity.ActivityMain;
import com.app.markeet.activity.ActivityNewsInfoDetails;
import com.app.markeet.activity.ActivityProductDetails;
import com.app.markeet.adapter.AdapterListener;
import com.app.markeet.adapter.AdapterHome;
import com.app.markeet.connection.API;
import com.app.markeet.connection.RestAdapter;
import com.app.markeet.connection.callbacks.RespFeaturedNews;
import com.app.markeet.connection.callbacks.RespProduct;
import com.app.markeet.data.ThisApp;
import com.app.markeet.databinding.FragmentHomeBinding;
import com.app.markeet.model.NewsInfo;
import com.app.markeet.model.Product;
import com.app.markeet.model.SortBy;
import com.app.markeet.utils.Tools;

import java.util.ArrayList;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class FragmentHome extends Fragment {

    private FragmentHomeBinding binding;
    private AdapterHome adapter;
    private boolean allLoaded = false;
    private Call callListing, callSlider;

    private List<NewsInfo> newsInfos = new ArrayList<>();
    private List<Product> products = new ArrayList<>();

    private boolean first_load = true;
    private boolean is_request_post_finish = false;
    private boolean is_request_slider_finish = false;

    public static FragmentHome instance() {
        return new FragmentHome();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        binding = FragmentHomeBinding.inflate(inflater, container, false);
        initComponent();

        requestAction(1, true);
        return binding.getRoot();
    }

    private void initComponent() {
        binding.recyclerView.setLayoutManager(new GridLayoutManager(getActivity(), Tools.getGridSpanCount(getActivity())));

        //set data and list adapter
        adapter = new AdapterHome(getActivity(), binding.recyclerView, AppConfig.general.product_per_request);
        binding.recyclerView.setAdapter(adapter);

        // detect when scroll reach bottom
        adapter.setListener(new AdapterListener<Object>() {
            @Override
            public void onClick(View view, String type, Object obj, int position) {
                super.onClick(view, type, obj, position);
                if (type.equals(AdapterHome.ActionType.ITEM.name())) {
                    Product product = (Product) obj;
                    ActivityProductDetails.navigate(getActivity(), product.id, false);
                } else if (type.equals(AdapterHome.ActionType.SLIDER.name())) {
                    NewsInfo newsInfo = (NewsInfo) obj;
                    ActivityNewsInfoDetails.navigate(getActivity(), newsInfo.id, false);
                }

                // show interstitial
                try {
                    ((ActivityMain) getActivity()).showInterstitialAd();
                } catch (Exception e) {
                }
            }

            @Override
            public void onLoadMore(int page) {
                super.onLoadMore(page);
                if (first_load && is_request_slider_finish && is_request_post_finish) {
                    requestAction(1, true);
                } else if (!first_load) {
                    if (allLoaded) {
                        adapter.setLoaded();
                    } else {
                        int next_page = page + 1;
                        requestAction(next_page, false);
                    }
                }
            }
        });

        binding.swipeRefresh.setOnRefreshListener(() -> {
            allLoaded = false;
            is_request_slider_finish = false;
            is_request_post_finish = false;
            products.clear();
            adapter.resetListData();
            requestAction(1, true);
        });
    }

    private void requestAction(final int page_no, boolean refresh) {
        first_load = refresh;
        showNoItemView(false);
        if (refresh) {
            if (callListing != null && callListing.isExecuted()) callListing.cancel();
            if (callSlider != null && callSlider.isExecuted()) callSlider.cancel();
            adapter.resetListData();
            swipeProgress(true);
            requestNewsInfo();
        } else {
            adapter.setLoadingOrFailed(null);
        }
        requestProduct(page_no);
    }

    private void requestNewsInfo() {
        API api = RestAdapter.createAPI();
        callSlider = api.getFeaturedNews();
        callSlider.enqueue(new Callback<RespFeaturedNews>() {
            @Override
            public void onResponse(Call<RespFeaturedNews> call, Response<RespFeaturedNews> response) {
                RespFeaturedNews resp = response.body();
                is_request_slider_finish = true;
                if (resp == null || resp.news_infos == null) return;
                newsInfos.clear();
                newsInfos.addAll(resp.news_infos);
                displayApiResult();
            }

            @Override
            public void onFailure(Call<RespFeaturedNews> call, Throwable t) {
                Log.e("onFailure", t.getMessage());
                if (!call.isCanceled()) onFailRequest();
            }

        });
    }

    private void requestProduct(Integer pageNo) {
        SortBy sort = ThisApp.get().getSorts().get(0);
        API api = RestAdapter.createAPI();
        callListing = api.getListProduct(pageNo, AppConfig.general.product_per_request, null, -1, sort.column, sort.order);
        callListing.enqueue(new Callback<RespProduct>() {
            @Override
            public void onResponse(Call<RespProduct> call, Response<RespProduct> response) {
                RespProduct resp = response.body();
                is_request_post_finish = true;
                allLoaded = resp.products.size() < AppConfig.general.product_per_request || resp.products.size() == 0;
                products.clear();
                products.addAll(resp.products);
                displayApiResult();
            }

            @Override
            public void onFailure(Call<RespProduct> call, Throwable t) {
                if (!call.isCanceled()) onFailRequest();
            }

        });
    }

    private void onFailRequest() {
        if (!(first_load && is_request_slider_finish && is_request_post_finish)) {
            return;
        }

        swipeProgress(false);
        if (Tools.isConnect(getActivity())) {
            adapter.setLoadingOrFailed(getString(R.string.failed_text));
        } else {
            adapter.setLoadingOrFailed(getString(R.string.no_internet_text));
        }
    }

    private void showNoItemView(boolean show) {
        ((TextView) binding.lytFailed.findViewById(R.id.failed_subtitle)).setText(getString(R.string.empty_state_no_data));
        if (show) {
            binding.lytFailed.setVisibility(View.VISIBLE);
        } else {
            binding.lytFailed.setVisibility(View.GONE);
        }
    }

    private void swipeProgress(final boolean show) {
        binding.swipeRefresh.post(() -> binding.swipeRefresh.setRefreshing(show));
        if (!show) {
            binding.recyclerView.setVisibility(View.VISIBLE);
            binding.shimmer.setVisibility(View.GONE);
            binding.shimmer.stopShimmer();
            return;
        }
        binding.recyclerView.setVisibility(View.GONE);
        binding.shimmer.setVisibility(View.VISIBLE);
        binding.shimmer.startShimmer();
    }

    private void displayApiResult() {
        if (first_load && is_request_slider_finish && is_request_post_finish) {
            first_load = false;
            is_request_slider_finish = false;
            is_request_post_finish = false;
            adapter.setSlider(newsInfos);
            adapter.insertData(products);
            showNoItemView(adapter.getItemCount() == 0);
            swipeProgress(false);
        } else if (!first_load) {
            adapter.insertData(products);
        }
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public void onResume() {
        super.onResume();
    }
}