package com.app.markeet.fragment;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.GridLayoutManager;

import com.app.markeet.AppConfig;
import com.app.markeet.R;
import com.app.markeet.activity.ActivityMain;
import com.app.markeet.activity.ActivityNewsInfoDetails;
import com.app.markeet.adapter.AdapterListener;
import com.app.markeet.adapter.AdapterNews;
import com.app.markeet.connection.API;
import com.app.markeet.connection.RestAdapter;
import com.app.markeet.connection.callbacks.RespNewsInfo;
import com.app.markeet.databinding.FragmentCategoryBinding;
import com.app.markeet.model.NewsInfo;
import com.app.markeet.utils.Tools;

import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class FragmentNewsInfo extends Fragment {

    public static FragmentNewsInfo instance() {
        return new FragmentNewsInfo();
    }

    private FragmentCategoryBinding binding;
    private AdapterNews adapter;
    private Call<RespNewsInfo> callbackCall;
    private boolean allLoaded = false;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        binding = FragmentCategoryBinding.inflate(inflater, container, false);
        initComponent();

        requestAction(1, true);
        return binding.getRoot();
    }

    private void initComponent() {
        binding.recyclerView.setLayoutManager(new GridLayoutManager(getActivity(), 1));

        //set data and list adapter
        adapter = new AdapterNews(getActivity(), binding.recyclerView, AppConfig.general.news_per_request);
        binding.recyclerView.setAdapter(adapter);

        // detect when scroll reach bottom
        adapter.setListener(new AdapterListener<NewsInfo>() {
            @Override
            public void onClick(View view, String type, NewsInfo obj, int position) {
                super.onClick(view, type, obj, position);
                try {
                    ActivityNewsInfoDetails.navigate(getActivity(), obj.id, false);
                    ((ActivityMain) getActivity()).showInterstitialAd();
                } catch (Exception e) {
                }
            }

            @Override
            public void onLoadMore(int page) {
                super.onLoadMore(page);
                if (allLoaded) {
                    adapter.setLoaded();
                } else {
                    int next_page = page + 1;
                    requestAction(next_page, false);
                }
            }
        });

        binding.swipeRefresh.setOnRefreshListener(() -> {
            adapter.resetListData();
            requestAction(1, true);
        });
    }

    private void requestAction(final int page_no, boolean refresh) {
        showNoItemView(false);
        if (refresh) {
            if (callbackCall != null && callbackCall.isExecuted()) callbackCall.cancel();
            adapter.resetListData();
            swipeProgress(true);
        } else {
            adapter.setLoadingOrFailed(null);
        }

        new Handler(Looper.getMainLooper()).postDelayed(() -> requestListNewsInfo(page_no), 500);

    }

    private void requestListNewsInfo(int page) {
        API api = RestAdapter.createAPI();
        callbackCall = api.getListNewsInfo(page, AppConfig.general.news_per_request, null);
        callbackCall.enqueue(new Callback<RespNewsInfo>() {
            @Override
            public void onResponse(Call<RespNewsInfo> call, Response<RespNewsInfo> response) {
                RespNewsInfo resp = response.body();
                if (resp != null && resp.status.equals("success")) {
                    allLoaded = resp.news_infos.size() < AppConfig.general.news_per_request || resp.news_infos.size() == 0;
                    displayApiResult(resp.news_infos);
                } else {
                    onFailRequest();
                }
            }

            @Override
            public void onFailure(Call<RespNewsInfo> call, Throwable t) {
                if (!call.isCanceled()) onFailRequest();
            }

        });
    }

    private void onFailRequest() {
        swipeProgress(false);
        if (Tools.isConnect(getActivity())) {
            adapter.setLoadingOrFailed(getString(R.string.failed_text));
        } else {
            adapter.setLoadingOrFailed(getString(R.string.no_internet_text));
        }
    }

    private void showNoItemView(boolean show) {
        ((TextView) binding.lytFailed.findViewById(R.id.failed_subtitle)).setText(getString(R.string.empty_state_no_data));
        if (show) {
            binding.lytFailed.setVisibility(View.VISIBLE);
        } else {
            binding.lytFailed.setVisibility(View.GONE);
        }
    }

    private void swipeProgress(final boolean show) {
        binding.swipeRefresh.post(() -> binding.swipeRefresh.setRefreshing(show));
        if (!show) {
            binding.recyclerView.setVisibility(View.VISIBLE);
            binding.shimmer.setVisibility(View.GONE);
            binding.shimmer.stopShimmer();
            return;
        }
        binding.recyclerView.setVisibility(View.GONE);
        binding.shimmer.setVisibility(View.VISIBLE);
        binding.shimmer.startShimmer();
    }

    private void displayApiResult(final List<NewsInfo> items) {
        adapter.insertData(items);
        swipeProgress(false);
        showNoItemView(adapter.getItemCount() == 0);
    }


}
