package com.app.markeet.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.fragment.app.Fragment;

import com.app.markeet.AppConfig;
import com.app.markeet.activity.ActivityInstruction;
import com.app.markeet.activity.ActivityNotification;
import com.app.markeet.activity.ActivityOrderHistory;
import com.app.markeet.activity.ActivityShoppingCart;
import com.app.markeet.activity.ActivityWishlist;
import com.app.markeet.data.DatabaseHandler;
import com.app.markeet.databinding.FragmentProfileBinding;
import com.app.markeet.utils.Tools;


public class FragmentProfile extends Fragment {

    private FragmentProfileBinding binding;
    private DatabaseHandler db;

    public static FragmentProfile instance() {
        return new FragmentProfile();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentProfileBinding.inflate(inflater, container, false);
        db = new DatabaseHandler(getActivity());
        initComponent();
        return binding.getRoot();
    }

    private void initComponent() {
        binding.actionShoppingCart.setOnClickListener(view -> {
            ActivityShoppingCart.navigate(getActivity());
        });

        binding.actionHistory.setOnClickListener(view -> {
            ActivityOrderHistory.navigate(getActivity());
        });

        binding.actionWhishlist.setOnClickListener(view -> {
            ActivityWishlist.navigate(getActivity());
        });

        binding.actionNotification.setOnClickListener(view -> {
            ActivityNotification.navigate(getActivity());
        });

        binding.actionInstruction.setOnClickListener(view -> {
            ActivityInstruction.navigate(getActivity());
        });

        binding.actionContactUs.setOnClickListener(view -> {
            Tools.directLinkToCustomTabs(getActivity(), AppConfig.general.contact_us_url);
        });

        binding.actionRate.setOnClickListener(view -> {
            Tools.rateAction(getActivity());
        });

        binding.actionAboutApp.setOnClickListener(view -> {
            Tools.showDialogAbout(getActivity());
        });

        binding.version.setText(Tools.getVersionName(getActivity()));
    }

    private void updateNavCounter() {
        int cart_count = db.getActiveCartSize();
        binding.cartCount.setText(String.valueOf(cart_count));

        // update wishlist counter
        int wishlist_count = db.getWishlistSize();
        binding.whishlistCount.setText(String.valueOf(wishlist_count));

        // update notification counter
        int notif_count = db.getUnreadNotificationSize();
        if (notif_count > 0) {
            binding.read.setVisibility(View.VISIBLE);
        } else {
            binding.read.setVisibility(View.GONE);
        }

    }

    @Override
    public void onResume() {
        super.onResume();
        updateNavCounter();
    }
}
